/* sane library:  enforces "zero-null" memory allocation
   Copyright (C) 2023  Terence Kelly
   See "Drill Bits" column in ACM _Queue_ magazine, Jan/Feb 2023.
   _ _ _ _ _ _ _ _ _ _

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.
   _ _ _ _ _ _ _ _ _ _

   Simple wrappers for allocation functions that map any standard C
   implementation from C89 thru C23 to the original intent of ANSI C
   as documented in the C89 and C99 Rationale documents: Zero-length
   allocation requests return NULL; realloc(p,0) means unconditional
   free(p), returns NULL, and cannot fail.  Wrapper for calloc checks
   for overflow and returns NULL.  Use with standard free().

   Exercise:  Implement these wrappers as macros that expand into
   expressions, which might be more efficient because it would
   eliminate function-call overhead.  Hints: Consider the ternary
   ("?:") operator.  For the free() in sane_realloc, consider the
   comma operator.  Should zero-length allocation requests set errno
   to ESILLY?
*/

#if ! defined __STDC__ || defined  __cplusplus
#error "compile as standard C only"
#endif

#ifdef SANE_STANDARDIZE
#error "SANE_STANDARDIZE macros are for callers only"
#endif

#include <stdint.h>  /* for SIZE_MAX */
#include <stdlib.h>  /* for malloc() etc. */
#include "sane.h"

void * sane_malloc(size_t size) {
  if (0 == size)  return NULL;
  else            return malloc(size);
}

void * sane_calloc(size_t nmemb, size_t size) {
  if (0 == nmemb || 0 == size || nmemb > SIZE_MAX / size)
       return NULL;
  else return calloc(nmemb, size);
}

void * sane_realloc(void *ptr, size_t size) {
  if      (NULL == ptr)            return sane_malloc(size);
  else if (0 == size) { free(ptr); return NULL; }
  else                             return realloc(ptr, size);
}

